﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using System.Collections;

namespace Kooboo.Web.Mvc.Grid
{
    public class GridModel
    {
        public GridModel(Type modelType, IEnumerable dataSource)
        {
            this.ModelMetadata = ModelMetadataProviders.Current.GetMetadataForType(() => null, modelType);

            DataSource = dataSource;


            GridColumns = GetColumns(ModelMetadata);
            GridActions = GetActions(ModelMetadata);
            GridItems = GetItems(this.DataSource, GridColumns, GridActions);

        }

        public ModelMetadata ModelMetadata { get; private set; }
        public IEnumerable DataSource { get; private set; }
        public IEnumerable<GridColumn> GridColumns { get; private set; }
        public IEnumerable<GridItem> GridItems { get; private set; }
        public IEnumerable<GridAction> GridActions { get; private set; }

        protected virtual IEnumerable<GridItem> GetItems(IEnumerable dataSource, IEnumerable<GridColumn> columns, IEnumerable<GridAction> gridActions)
        {
            List<GridItem> items = new List<GridItem>();

            if (dataSource != null)
            {
                int i = 0;
                foreach (var dataItem in dataSource)
                {
                    items.Add(new GridItem(i, dataItem, columns, gridActions));
                }
            }

            return items;
        }
        protected virtual IEnumerable<GridAction> GetActions(ModelMetadata modelMetadata)
        {
            List<GridAction> actions = new List<GridAction>();
            if (modelMetadata is CustomModelMetadata)
            {
                foreach (var item in ((CustomModelMetadata)modelMetadata).Attributes.OfType<GridActionAttribute>().OrderBy(it => it.Order))
                {
                    var gridAction = new GridAction()
                    {
                        ActionName = item.ActionName,
                        ControllerName = item.ControllerName,
                        DisplayName = item.DisplayName ?? item.ActionName,
                        ConfirmMessage = item.ConfirmMessage,
                        VisibleProperty = item.VisibleProperty
                    };

                    if (!string.IsNullOrEmpty(item.RouteValueProperties))
                    {
                        List<GridActionRouteValuesSetting> settingList = new List<GridActionRouteValuesSetting>();
                        foreach (var property in item.RouteValueProperties.Split(','))
                        {
                            var setting = ParseRouteValueSetting(property);
                            if (setting != null)
                            {
                                settingList.Add(setting);
                            }
                        }
                        gridAction.RouteValuesSetting = settingList;
                    }
                    if (item.VisibleArbiter != null)
                    {
                        gridAction.VisibleArbiter = (IGridActionVisibleArbiter)Activator.CreateInstance(item.VisibleArbiter);
                    }
                    if (item.RouteValuesGetter != null)
                    {
                        gridAction.RouteValuesGetter = (IGridActionRouteValuesGetter)Activator.CreateInstance(item.RouteValuesGetter);
                    }

                    actions.Add(gridAction);
                }
            }
            else
            {
                //other ModelMetadata type....
            }
            return actions;
        }
        private static GridActionRouteValuesSetting ParseRouteValueSetting(string setting)
        {
            if (string.IsNullOrEmpty(setting))
            {
                return null;
            }
            var arr = setting.Split('=');
            GridActionRouteValuesSetting routeValuesSetting = new GridActionRouteValuesSetting();
            if (arr.Length > 1)
            {
                routeValuesSetting.RouteValueName = arr[0];
                routeValuesSetting.PropertyName = arr[1];
            }
            else
            {
                routeValuesSetting.RouteValueName = arr[0].Replace('.', '_');
                routeValuesSetting.PropertyName = arr[0];
            }
            return routeValuesSetting;
        }
        protected virtual IEnumerable<GridColumn> GetColumns(ModelMetadata modelMetadata)
        {
            var gridColumns = new List<GridColumn>();
            foreach (var propertyMetadata in modelMetadata.Properties)
            {
                var column = GetGridColumn(propertyMetadata);
                if (column != null)
                {
                    gridColumns.Add(column);
                }
            }
            return gridColumns.OrderBy(go => go.Order);
        }
        protected virtual GridColumn GetGridColumn(ModelMetadata propertyMetadata)
        {
            if (ModelMetadata is CustomModelMetadata)
            {
                var attribute = ((CustomModelMetadata)propertyMetadata).Attributes.OfType<GridColumnAttribute>().FirstOrDefault();
                if (attribute != null)
                {
                    GridColumn gridColumn = new GridColumn();
                    gridColumn.PropertyName = propertyMetadata.PropertyName;
                    gridColumn.HeaderText = attribute.HeaderText ?? propertyMetadata.GetDisplayName();
                    gridColumn.HeaderFormatString = attribute.HeaderFormatString;
                    gridColumn.ItemFormatString = attribute.ItemFormatString ?? propertyMetadata.DisplayFormatString;
                    gridColumn.Order = attribute.Order;
                    if (attribute.HeaderRenderType != null)
                    {
                        gridColumn.HeaderRender = (IColumnHeaderRender)Activator.CreateInstance(attribute.HeaderRenderType);
                    }
                    if (attribute.ItemRenderType != null)
                    {
                        gridColumn.ItemRender = (IItemColumnRender)Activator.CreateInstance(attribute.ItemRenderType);
                    }
                    if (attribute.AlternatingItemRenderType != null)
                    {
                        gridColumn.AlternatingItemRender = (IItemColumnRender)Activator.CreateInstance(attribute.AlternatingItemRenderType);
                    }
                    return gridColumn;

                }
                return null;
            }
            else
            {
                //get from othe ModelMetadata type.

                return null;
            }
        }
    }
}
